@@ -1,4 +1,4 @@
-/* $Id: acconfig.h,v 1.2.2.8 1996/07/15 16:04:35 wessels Exp $ */
+/* $Id: acconfig.h,v 1.10 1997/01/07 20:05:31 wessels Exp $ */
 
 /* Define if struct tm has tm_gmtoff member */
 #undef HAVE_TM_GMTOFF
@@ -7,7 +7,7 @@
 #undef HAVE_EXT_MALLINFO
 
 /* Maximum number of open filedescriptors */
-#undef SQUID_FD_SETSIZE
+#undef SQUID_MAXFD
 
 /* UDP send buffer size */
 #undef SQUID_UDP_SO_SNDBUF
@@ -29,3 +29,15 @@
 
 /* If gettimeofday is known to take only one argument */
 #undef GETTIMEOFDAY_NO_TZP
+
+/* If libresolv.a has been hacked to export _dns_ttl_ */
+#undef LIBRESOLV_DNS_TTL_HACK
+
+/* Define if struct ip has ip_hl member */
+#undef HAVE_IP_HL
+
+/* Define if your compiler supports prototyping */
+#undef HAVE_ANSI_PROTOTYPES
+
+/* Define if we should use GNU regex */
+#undef USE_GNUREGEX
