@@ -1,5 +1,5 @@
 /*
- * $Id: acl.c,v 1.76 1997/01/19 08:32:09 wessels Exp $
+ * $Id: acl.c,v 1.83 1997/02/06 19:59:53 wessels Exp $
  *
  * DEBUG: section 28    Access Control
  * AUTHOR: Duane Wessels
@@ -50,13 +50,26 @@ static struct _acl *AclList = NULL;
 static struct _acl **AclListTail = &AclList;
 static const char *const w_space = " \t\n\r";	/* Jasper sez so */
 
+
+#ifdef USE_SPLAY_TREE
+int aclCompIpResult = 0;
+static struct _acl_ip_data *aclSplayInsertIp
+             _PARAMS((struct _acl_ip_data *, struct _acl_ip_data *));
+static struct _acl_ip_data *aclSplayIp
+             _PARAMS((struct in_addr, struct _acl_ip_data *));
+static int aclSplayIpCompare
+    _PARAMS((struct in_addr addr, struct _acl_ip_data * data));
+static int aclMatchIp _PARAMS((void *dataptr, struct in_addr c));
+#else
+static int aclMatchIp _PARAMS((struct _acl_ip_data * data, struct in_addr c));
+#endif /* USE_SPLAY_TREE */
+
 static void aclDestroyAclList _PARAMS((struct _acl_list * list));
 static void aclDestroyIpList _PARAMS((struct _acl_ip_data * data));
 static void aclDestroyTimeList _PARAMS((struct _acl_time_data * data));
 static int aclMatchDomainList _PARAMS((wordlist *, const char *));
 static int aclMatchAclList _PARAMS((const struct _acl_list *, aclCheck_t *));
 static int aclMatchInteger _PARAMS((intlist * data, int i));
-static int aclMatchIp _PARAMS((struct _acl_ip_data * data, struct in_addr c));
 static int aclMatchTime _PARAMS((struct _acl_time_data * data, time_t when));
 static int aclMatchIdent _PARAMS((wordlist * data, const char *ident));
 static squid_acl aclType _PARAMS((const char *s));
@@ -193,6 +206,8 @@ aclParseMethodList(void *curlist)
     while ((t = strtokFile())) {
 	q = xcalloc(1, sizeof(intlist));
 	q->i = (int) urlParseMethod(t);
+	if (q->i == METHOD_PURGE)
+	    Config.Options.enable_purge = 1;
 	*(Tail) = q;
 	Tail = &q->next;
     }
@@ -265,9 +280,6 @@ aclParseIpData(const char *t)
 	q->mask.s_addr = 0;
 	return q;
     }
-    memset(addr1, 0, 256);
-    memset(addr2, 0, 256);
-    memset(mask, 0, 256);
     if (sscanf(t, "%[0-9.]-%[0-9.]/%[0-9.]", addr1, addr2, mask) == 3) {
 	(void) 0;
     } else if (sscanf(t, "%[0-9.]-%[0-9.]", addr1, addr2) == 2) {
@@ -317,6 +329,23 @@ aclParseIpData(const char *t)
     return q;
 }
 
+#ifdef USE_SPLAY_TREE
+
+static void
+aclParseIpList(void *curlist)
+{
+    char *t = NULL;
+    struct _acl_ip_data **ip_data = curlist;
+    struct _acl_ip_data *q = NULL;
+    while ((t = strtokFile())) {
+	if ((q = aclParseIpData(t)) == NULL)
+	    continue;
+	*ip_data = aclSplayInsertIp(q, *ip_data);
+    }
+}
+
+#else
+
 static void
 aclParseIpList(void *curlist)
 {
@@ -332,6 +361,116 @@ aclParseIpList(void *curlist)
     }
 }
 
+#endif
+
+#ifdef USE_SPLAY_TREE
+static int
+aclSplayIpCompare(struct in_addr addr, struct _acl_ip_data *data)
+{
+    int rc = 0;
+    addr.s_addr &= data->mask.s_addr;	/* apply netmask */
+    if (data->addr2.s_addr == 0) {	/* single address check */
+	if (addr.s_addr > data->addr1.s_addr)
+	    rc = 1;
+	else if (addr.s_addr < data->addr1.s_addr)
+	    rc = -1;
+	else
+	    rc = 0;
+    } else {			/* range address check */
+	if (addr.s_addr > data->addr2.s_addr)
+	    rc = 1;
+	else if (addr.s_addr < data->addr1.s_addr)
+	    rc = -1;
+	else
+	    rc = 0;
+    }
+    return rc;
+}
+
+static struct _acl_ip_data *
+aclSplayInsertIp(struct _acl_ip_data *q, struct _acl_ip_data *t)
+{
+    struct _acl_ip_data *new;
+    new = xmalloc(sizeof(struct _acl_ip_data));
+    new->addr1 = q->addr1;
+    new->addr2 = q->addr2;
+    new->mask = q->mask;
+    if (t == NULL) {
+	new->left = new->right = NULL;
+	return new;
+    }
+    t = aclSplayIp(q->addr1, t);
+    if (aclCompIpResult < 0) {
+	new->left = t->left;
+	new->right = t;
+	t->left = NULL;
+	return new;
+    } else if (aclCompIpResult > 0) {
+	new->right = t->right;
+	new->left = t;
+	t->right = NULL;
+	return new;
+    } else {
+	debug_trap("aclSplayInsertIp: entry duplicated\n");
+	safe_free(new);
+	return t;
+    }
+}
+
+static struct _acl_ip_data *
+aclSplayIp(struct in_addr addr1, struct _acl_ip_data *t)
+{
+    struct _acl_ip_data N;
+    struct _acl_ip_data *l;
+    struct _acl_ip_data *r;
+    struct _acl_ip_data *y;
+    if (t == NULL)
+	return t;
+    N.left = N.right = NULL;
+    l = r = &N;
+
+    for (;;) {
+	aclCompIpResult = aclSplayIpCompare(addr1, t);
+	if (aclCompIpResult < 0) {
+	    if (t->left == NULL)
+		break;
+	    if (aclSplayIpCompare(addr1, t->left) < 0) {
+		y = t->left;	/* rotate right */
+		t->left = y->right;
+		y->right = t;
+		t = y;
+		if (t->left == NULL)
+		    break;
+	    }
+	    r->left = t;	/* link right */
+	    r = t;
+	    t = t->left;
+	} else if (aclCompIpResult > 0) {
+	    if (t->right == NULL)
+		break;
+	    if (aclSplayIpCompare(addr1, t->right) > 0) {
+		y = t->right;	/* rotate left */
+		t->right = y->left;
+		y->left = t;
+		t = y;
+		if (t->right == NULL)
+		    break;
+	    }
+	    l->right = t;	/* link left */
+	    l = t;
+	    t = t->right;
+	} else {
+	    break;
+	}
+    }
+    l->right = t->left;		/* assemble */
+    r->left = t->right;
+    t->left = N.right;
+    t->right = N.left;
+    return t;
+}
+#endif /* USE_SPLAY_TREE */
+
 static void
 aclParseTimeSpec(void *curlist)
 {
@@ -501,7 +640,8 @@ aclParseAclLine(void)
 	debug(28, 0, "aclParseAclLine: Invalid ACL type '%s'\n", t);
 	return;
     }
-    if ((A = aclFindByName(t)) == NULL) {
+    if ((A = aclFindByName(aclname)) == NULL) {
+	debug(28, 3, "aclParseAclLine: Creating ACL '%s'\n", aclname);
 	A = xcalloc(1, sizeof(struct _acl));
 	xstrncpy(A->name, aclname, ACL_NAME_SZ);
 	A->type = acltype;
@@ -513,12 +653,12 @@ aclParseAclLine(void)
 	    debug(28, 0, "aclParseAclLine: ACL '%s' already exists with different type, skipping.\n", A->name);
 	    return;
 	}
-	debug(28, 0, "aclParseAclLine: Appending to '%s'\n", aclname);
+	debug(28, 3, "aclParseAclLine: Appending to '%s'\n", aclname);
     }
     switch (A->type) {
     case ACL_SRC_IP:
     case ACL_DST_IP:
-	aclParseIpList(&(A->data));
+	aclParseIpList(&A->data);
 	break;
     case ACL_SRC_DOMAIN:
     case ACL_DST_DOMAIN:
@@ -694,8 +834,13 @@ aclParseAccessLine(struct _acl_access **head)
 }
 
 static int
+#ifndef USE_SPLAY_TREE
 aclMatchIp(struct _acl_ip_data *data, struct in_addr c)
+#else
+aclMatchIp(void *dataptr, struct in_addr c)
+#endif				/* USE_SPLAY_TREE */
 {
+#ifndef USE_SPLAY_TREE
     struct in_addr h;
     unsigned long lh, la1, la2;
     struct _acl_ip_data *first, *prev;
@@ -734,6 +879,11 @@ aclMatchIp(struct _acl_ip_data *data, struct in_addr c)
     }
     debug(28, 3, "aclMatchIp: returning 0\n");
     return 0;
+#else
+    struct _acl_ip_data **data = dataptr;
+    *data = aclSplayIp(c, *data);
+    return !aclCompIpResult;
+#endif /* USE_SPLAY_TREE */
 }
 
 static int
@@ -862,14 +1012,22 @@ aclMatchAcl(struct _acl *acl, aclCheck_t * checklist)
     debug(28, 3, "aclMatchAcl: checking '%s'\n", acl->cfgline);
     switch (acl->type) {
     case ACL_SRC_IP:
+#ifndef USE_SPLAY_TREE
 	return aclMatchIp(acl->data, checklist->src_addr);
+#else
+	return aclMatchIp(&acl->data, checklist->src_addr);
+#endif /* USE_SPLAY_TREE */
 	/* NOTREACHED */
     case ACL_DST_IP:
 	ia = ipcache_gethostbyname(r->host, IP_LOOKUP_IF_MISS);
 	if (ia) {
 	    for (k = 0; k < (int) ia->count; k++) {
 		checklist->dst_addr = ia->in_addrs[k];
+#ifndef USE_SPLAY_TREE
 		if (aclMatchIp(acl->data, checklist->dst_addr))
+#else
+		if (aclMatchIp(&acl->data, checklist->dst_addr))
+#endif /* USE_SPLAY_TREE */
 		    return 1;
 	    }
 	    return 0;
@@ -879,7 +1037,11 @@ aclMatchAcl(struct _acl *acl, aclCheck_t * checklist)
 	    checklist->state[ACL_DST_IP] = ACL_LOOKUP_NEED;
 	    return 0;
 	} else {
+#ifndef USE_SPLAY_TREE
 	    return aclMatchIp(acl->data, no_addr);
+#else
+	    return aclMatchIp(&acl->data, no_addr);
+#endif /* USE_SPLAY_TREE */
 	}
 	/* NOTREACHED */
     case ACL_DST_DOMAIN:
@@ -978,6 +1140,19 @@ aclCheck(const struct _acl_access *A, aclCheck_t * checklist)
     return !allow;
 }
 
+#ifdef USE_SPLAY_TREE
+static void
+aclDestroyIpList(struct _acl_ip_data *data)
+{
+    if (data == NULL)
+	return;
+    aclDestroyIpList(data->left);
+    aclDestroyIpList(data->right);
+    safe_free(data);
+}
+
+#else
+
 static void
 aclDestroyIpList(struct _acl_ip_data *data)
 {
@@ -987,6 +1162,7 @@ aclDestroyIpList(struct _acl_ip_data *data)
 	safe_free(data);
     }
 }
+#endif
 
 static void
 aclDestroyTimeList(struct _acl_time_data *data)
