@@ -1,5 +1,6 @@
+
 /*
- * $Id: acl.h,v 1.32 1997/01/14 19:50:06 wessels Exp $
+ * $Id: acl.h,v 1.34 1997/02/06 19:59:54 wessels Exp $
  *
  * AUTHOR: Duane Wessels
  *
@@ -61,7 +62,11 @@ struct _acl_ip_data {
     struct in_addr addr1;	/* if addr2 non-zero then its a range */
     struct in_addr addr2;
     struct in_addr mask;
+#if USE_SPLAY_TREE
+    struct _acl_ip_data *left, *right;
+#else
     struct _acl_ip_data *next;
+#endif
 };
 
 struct _acl_time_data {
