@@ -1,6 +1,6 @@
 
 /*
- * $Id: pass.c,v 1.22 1997/01/21 16:42:05 wessels Exp $
+ * $Id: pass.c,v 1.26 1997/02/05 04:54:13 wessels Exp $
  *
  * DEBUG: section 39    HTTP Passthrough
  * AUTHOR: Duane Wessels
@@ -590,21 +590,23 @@ passSelectNeighbor(int u1, const ipcache_addrs * ia, void *data)
 {
     PassStateData *passState = data;
     request_t *request = passState->request;
-    edge *e = NULL;
-    edge *g = NULL;
+    peer *e = NULL;
+    peer *g = NULL;
     int fw_ip_match = IP_ALLOW;
     if (ia && Config.firewall_ip_list)
 	fw_ip_match = ip_access_check(ia->in_addrs[ia->cur], Config.firewall_ip_list);
-    if ((e = Config.passProxy)) {
-	hierarchyNote(request, HIER_PASS_PARENT, 0, e->host);
-    } else if (matchInsideFirewall(request->host)) {
+    if (matchInsideFirewall(request->host)) {
 	hierarchyNote(request, HIER_DIRECT, 0, request->host);
     } else if (fw_ip_match == IP_DENY) {
-	hierarchyNote(request, HIER_DIRECT, 0, request->host);
+	hierarchyNote(request, HIER_FIREWALL_IP_DIRECT, 0, request->host);
+    } else if ((e = Config.passProxy)) {
+	hierarchyNote(request, HIER_PASS_PARENT, 0, e->host);
     } else if ((e = getDefaultParent(request))) {
 	hierarchyNote(request, HIER_DEFAULT_PARENT, 0, e->host);
     } else if ((e = getSingleParent(request))) {
 	hierarchyNote(request, HIER_SINGLE_PARENT, 0, e->host);
+    } else if ((e = getRoundRobinParent(request))) {
+	hierarchyNote(request, HIER_ROUNDROBIN_PARENT, 0, e->host);
     } else if ((e = getFirstUpParent(request))) {
 	hierarchyNote(request, HIER_FIRSTUP_PARENT, 0, e->host);
     }
